local options = {
	type = "group",
	name = "EggTimer",
	handler = EggTimer,
	get = function(info) return EggTimer.db[ info[#info] ] end,
	set = function(info, value) EggTimer.db[ info[#info] ] = value end,
	args = {
		General = {
			order = 1,
			type = "group",
			name = "General Settings",
			desc = "General Settings",
			args = {
				intro = {
					order = 1,
					type = "description",
					name = "Monitors the time left on your Mysterious Egg and other perishable items",
				},
				MinimapIcon = {
					type = "toggle",
					name = "Show minimap button",
					desc = "Show the EggTimer minimap button",
					order = 2,
					get = function(info) return not EggTimer.db.MinimapIcon.hide end,
					set = function(info, value)
						EggTimer.db.MinimapIcon.hide = not value
						if value then LDBIcon:Show("EggTimer") else LDBIcon:Hide("EggTimer") end
					end,
					hidden = function() return not LDBIcon end,
				},
			},
		},
		Calendar = {
			order = 2,
			type = "group",
			name = "Blizzard Calendar",
			desc = "Configure use of the in-game calendar",
			args = {
				intro = {
					order = 1,
					type = "description",
					name = "Configure use of the in-game calendar",
				},
				enable = {
					order = 2,
					type = "toggle",
					name = "Enable Calendar events",
					desc = "Create new Blizzard Calendar events automatically",
					get = function(info) return EggTimer.db.Calendar.Enable end,
					set = function(info, v)
						EggTimer.db.Calendar.Enable = v
						return EggTimer.db.Calendar.Enable
					end,
				},
			},
		},
		Bar = {
			order = 3,
			type = "group",
			name = "Bar Settings",
			desc = "Configure timer bar display",
			args = {
				intro = {
					order = 1,
					type = "description",
					name = "Configure timer bar display",
				},
			},
		},
		Tooltip = {
			order = 4,
			type = "group",
			name = "Tooltip Settings",
			desc = "Select the contents of the tooltip",
			args = {
				intro = {
					order = 1,
					type = "description",
					name = "Select the contents of the tooltip",
				},
			},
		},
		Announce = {
			order = 5,
			type = "group",
			name = "Announcements",
			desc = "Choose your notification method",
			get = function(info) return EggTimer.db.Announce[ info[#info] ] end,
			set = function(info, value)
				EggTimer.db.Announce[ info[#info] ] = value
			end,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = "Choose your notification method",
				},
				Sound = {
					type = "toggle",
					order = 2,
					name = "Enable Sound",
					desc = "Causes EggTimer to play a chosen sound effect",
				},
				Message = {
					type = "toggle",
					order = 3,
					name = "Enable Announcement",
					desc = "Print a message to screen when items expire",
				},
				Output = EggTimer:GetSinkAce3OptionsDataTable(),
				SoundFile = {
					type = "select", dialogControl = 'LSM30_Sound',
					order = 5,
					name = "Sound to play",
					desc = "Sound to play",
					values = AceGUIWidgetLSMlists.sound,
					disabled = function() return not EggTimer.db.Announce.Sound end,
				},
				test = {
					type = "execute",
					order = -1,
					name = "Test announcement",
					desc = "Test announcement",
					func = function()
						EggTimer:Announce(EggTimer.db.Announce.Sound, EggTimer.db.Announce.Message and "Test announcement")
					end,
				},
			},
		},
	},
}
EggTimer.Options = options
